﻿#-*- coding: utf-8 -*-
import sys
import os

def getGeo_GoogleLocApi(ip=None, lat=0.0, long=0.0, proxy=None):
  gCode = dict()

  print(lat, long)
  gCode['latitude'] = lat
  gCode['longitude'] = long
  gCode['address'] =  u'';

  #return gCode
  if ip == 'local':
    pass
  else:
    import urllib2
    import ujson as json

    # infodb
    # http://api.ipinfodb.com/v3/ip-city/?key=<your_api_key>&ip=74.125.45.100&format=json
    # http://api.ipinfodb.com/v3/ip-city/?key=<your_api_key>&ip=74.125.45.100&format=json
    API_KEY_IPINFODB = "<place your api key to here>"
    url_infodb = "http://api.ipinfodb.com/v3/ip-city/?key=%s&format=json" % API_KEY_IPINFODB
    #print(url_infodb)

    # google
    url_google_address  = "http://maps.googleapis.com/maps/api/geocode/json?latlng={0},{1}&sensor=true&language=zh-CN"
    url_google_location = "http://maps.googleapis.com/maps/api/geocode/json?address={0}&sensor=true&language=zh-CN&components=country:CN"

    response = urllib2.urlopen(url_infodb)
    geocode = json.decode(response.read())
    #print(geocode)
    if geocode["statusCode"] == "OK":
      lat = geocode["latitude"]
      lng = geocode["longitude"]
    else:
      lat = lat
      lng = long

    gCode['latitude'] = float(lat)
    gCode['longitude'] = float(lng)

    #print(url_google_address.format(lat, lng))
    response = urllib2.urlopen(url_google_address.format(lat, lng))
    geocode = json.decode(response.read())
    if geocode["status"] == "OK":
      gCode['address'] = geocode["results"][0]["formatted_address"]
      print(gCode['address'])
    else:
      gCode['address'] = ""

  return(gCode)

def calcSunsetSunRise(lat, long, date_begin, date_end):
  import ephem
  import datetime

  obs = ephem.Observer()
  obs.lat = str(lat)
  obs.long= str(long)

  td = datetime.timedelta(days=1)

  y = date_begin // 10000
  m = (date_begin % 10000) // 100
  d = date_begin % 100
  start_date = datetime.datetime(y, m, d) - td

  y = date_end // 10000
  m = (date_end % 10000) // 100
  d = date_end % 100
  end_date = datetime.datetime(y, m, d)

  del y, m, d

  sun = ephem.Sun()

  sunrises = []
  sunsets = []
  dates = []

  date = start_date
  while date < end_date:
    date += td
    dates.append(date)
    obs.date = date

    rise_time = obs.next_rising(sun).datetime()
    sunrises.append(rise_time)

    set_time = obs.next_setting(sun).datetime()
    sunsets.append(set_time)

  del start_date, end_date, td
  del date, rise_time, set_time
  del obs, sun
  return(dates, sunrises, sunsets)
  pass

def calcTimeZone(long):
  import math
  return math.ceil(long / 15)
  pass

def genBrightnessCurve(interval=5, time_s=0630, time_e=1830, min=25, max=90):
  import datetime
  import math

  curve = []

  td = datetime.timedelta(minutes=interval)

  h, m = time_s // 100, time_s % 100
  stime = datetime.datetime(1, 1, 1, h, m, 0)

  h, m = time_e // 100, time_e % 100
  etime = datetime.datetime(1, 1, 1, h, m, 0)

  count = (etime - stime).seconds / 60 // interval

  #print((etime - stime).seconds)
  #print(count)

  w = 180.0 / count
  d = math.pi / 180
  d = math.pi / count
  a = max - min
  for t in xrange(0, count):
    #curve.append(a * math.sin(d * ( w * t) ))
    curve.append(a * math.sin(d * t) + min)

  for x in xrange(0, count):
    if curve[x] < min:
      curve[x] = min
    elif curve[x] > max:
      curve[x] = max
    else:
      curve[x] = int(curve[x])

  return(curve)


def QueryInfo(ip=None, lat=0.0, long=0.0, interval=5, min=15, max=85, proxy=None):
  import datetime

  #print(lat, long, proxy)
  gCode = getGeo_GoogleLocApi(ip=ip, lat=lat, long=long, proxy=proxy)
  #print(gCode)

  lat = gCode['latitude']
  long = gCode['longitude']
  if gCode.has_key('address'):
    loc = gCode['address']
  else:
    loc = u''

  del gCode

  today = datetime.datetime.today()
  td = datetime.timedelta(days=7)
  nxday = today + td

  sday = int('%04d%02d%02d' % (today.year, today.month, today.day))
  eday = int('%04d%02d%02d' % (nxday.year, nxday.month, nxday.day))

  del today, td, nxday

  tz = calcTimeZone(long)

  date, rise, sets = calcSunsetSunRise(lat, long, sday, eday)

  vs = []
  for c in xrange(len(date)):
    td = datetime.timedelta(hours=tz)
    s_date = (date[c] + td).date().isoformat()
    s_rise = (rise[c] + td).time().isoformat()
    s_sets = (sets[c] + td).time().isoformat()

    vs.append((s_date, s_rise, s_sets))
    #print(s_date, s_rise, s_sets)

  #
  #
  time_s = int(vs[0][1][0:2])*100+int(vs[0][1][3:5])
  time_e = int(vs[0][2][0:2])*100+int(vs[0][2][3:5])
  curve = genBrightnessCurve(interval=interval, time_s=time_s, time_e=time_e, min=min, max=max)

  if __name__ == "__main__":
    print(lat, long, loc.encode('utf8'), vs)
    print(curve)
  else:
    #print(lat, long, loc.encode('gbk'), vs)
    UpdateData(lat, long, loc, vs)
    UpdateCurve(curve)
    #print(curve)
    pass

  del lat, long, loc
  del sday, eday
  del s_date, s_rise, s_sets
  del tz, date, rise, sets
  del vs, curve
  pass

def testGeo():
  gCode = getGeo_GoogleLocApi()
  for g in gCode:
    if type(gCode[g]) == type(1.0):
      print(u"%10s : %12.6f" % (g, gCode[g]))
    else:
      print(u"%10s : %s" % (g, gCode[g].decode("utf8")))
  del g, gCode

  pass

def testSun():
  sun = calcSunsetSunRise(18.26, 109.50, 20110601, 20110607)
  print(sun)
  del sun

  pass


if __name__ == "__main__":
  import gc
  # Enable automatic garbage collection.
  gc.enable()
  # Set the garbage collection debugging flags.
  gc.set_debug(gc.DEBUG_COLLECTABLE | gc.DEBUG_UNCOLLECTABLE | gc.DEBUG_INSTANCES | gc.DEBUG_OBJECTS)

  QueryInfo()
  #QueryInfo(proxy="127.0.0.1:8118")

  #testGeo()
  #testSun()

  print 'begin collect...'
  _unreachable = gc.collect()
  print 'unreachable object num:%d' % _unreachable
  print 'garbage object num:%d' % len(gc.garbage)

  #print(gc.garbage)
  #raw_input("Pressing any key to continue...")
  pass

else:
  from AutoBrightness import *
  pass

